#!/usr/bin/env python3
# eqmerge.py
# Copyright 2015-2016
import sys, struct, argparse, time


Description = """Tool to merge ordering/delta components produced by eqsplit.py"""

def merge(ofile, dfile, wsize):
  assert(wsize>2 and wsize<=128)
  outname = ofile + ".merge"
  out = open(outname,"wb")
  buffer = []     # init buffer as in the coding procedre
  with open(ofile,"rb") as fo:
    with open(dfile,"rb") as fd: 
      while True:
        # get 1 byte from order file
        z = fo.read(1)
        if len(z)!=1:
          break
        code = struct.unpack('B', z)[0]   # decode a single byte
        if len(buffer)==0 or code%2==1:
          # get 4 bytes from value file
          s = fd.read(4)
          if len(s)==0:
            raise Exception("Illegal delta file (too short)")
          elif len(s)!=4:
            raise Exception("Illegal value file (not 32bit ints)")          
          if len(buffer)==0:                # first entry only
            v = struct.unpack('<i', s)[0]   # decode a 4-byte int    
          else:  
            delta = struct.unpack('<I', s)[0]   # decode a 4-byte uint
            # decode the next value
            assert 0 < code < 2*wsize, "Illegal code in file: %d" % code
            if code == 1:
              v = min(buffer)-delta       # v is smaller than anyone in the buffer 
            else:
              v =  buffer[-(code//2)]+ delta   # add delta to the predecessor
        else:  # case not first value and code is even
          v = buffer[-(code//2)]  # v is equal to a previous value in the buffer
        assert(v>= -pow(2,31) and v<pow(2,31) )  
        # write the value to the buffer and output file
        buffer.append(v)
        if len(buffer)>= wsize:
          assert len(buffer)==wsize
          buffer.pop(0)
        out.write(struct.pack('i',v))
  out.close()
  


def main():
  # show_command_line(sys.stderr)
  parser = argparse.ArgumentParser(description=Description, formatter_class=argparse.RawTextHelpFormatter)
  parser.add_argument('infile', help='order file', type=str)
  args = parser.parse_args()  
  # run the algorithm   
  s1 = time.time()
  ofile = args.infile
  dfile = ofile[:-1] + "d"
  wsize = int(ofile.split(".")[-2])
  sys.stderr.write("Order file %s, delta file %s, window size %d\n" %(ofile,dfile,wsize))
  merge(ofile,dfile,wsize)
  e1 = time.time()
  sys.stderr.write("Elapsed time: %.3f\n" % (e1-s1))

  
if __name__ == '__main__':
    main()
 
